local AceGCInfoVar = {}
AceGCInfoVar.DEFAULT_OPTIONS = {
	Locked = TRUE,
}

local gnome = Metrognome:GetInstance("1")

AceGCInfo = AceAddonClass:new({
	name		= ACEGCINFO.NAME,
	description	= ACEGCINFO.DESCRIPTION,
	version		= ACEGCINFO.VERSION,
	releaseDate	= "06/05/06",
	aceCompatible	= "102",
	author		= "Phyber of Bloodhoof",
	category	= "bars",
	db		= AceDatabase:new("AceGCInfoDB"),
	defaults	= AceGCInfoVar.DEFAULT_OPTIONS,
	cmd		= AceChatCmdClass:new(ACEGCINFO.COMMANDS, ACEGCINFO.CMD_OPTIONS)
})

if FuBar then
	AceGCInFu = FuBarPlugin:GetInstance("1.2"):new({
		name		= "AceGCInFu",
		description	= "AceGCInfo via FuBar",
		version		= "3",
		releaseDate	= "07/06/2006",
		aceCompatible	= 103,
		author		= "Phyber",
		website		= "http://phyber.wowinterface.com/",
		category	= "others",
		db		= AceDatabase:new("AceGCInFuDB"),
		--hasIcon		= "Interface\\AddOns\\AceGCInfo\\AceGCInFu.tga",
		clickableTooltip = true,
	})
end

function AceGCInfo:Initialize()
	self.Msg = function(...) self.cmd:result(format(unpack(arg))) end
	self.TogOpt = function(var) return self.db:toggle(self.profilePath,var) end
	self.GetOpt = function(var) return self.db:get(self.profilePath,var) end
	self.SetOpt = function(var,val) self.db:set(self.profilePath,var,val) end

	self:MakeFrame()
end

-- Create the frame
function AceGCInfo:MakeFrame()
	AceGCInfoFrame = CreateFrame("Frame", "AceGCInfoFrame", UIParent)
	AceGCInfoFrame:SetFrameStrata("MEDIUM")
	AceGCInfoFrame:EnableMouse(true)
	AceGCInfoFrame:SetMovable(true)
	AceGCInfoFrame:SetWidth(420)
	AceGCInfoFrame:SetHeight(26)

	AceGCInfoFrame:SetBackdrop({
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = {left = 5, right = 5, top = 5, bottom = 5},
	})

	AceGCInfoFrame:RegisterForDrag("LeftButton")
	AceGCInfoFrame:SetScript("OnDragStart", function() if not self.GetOpt("Locked") then AceGCInfoFrame:StartMoving() end end)
	AceGCInfoFrame:SetScript("OnDragStop", function() AceGCInfoFrame:StopMovingOrSizing() self:SavePosition() end)
	
	local AceGCInfoText = AceGCInfoFrame:CreateFontString("AceGCInfoText", "ARTWORK")
	AceGCInfoText:SetFontObject(GameFontNormal)
	AceGCInfoText:SetPoint("CENTER", AceGCInfoFrame, "CENTER")
	AceGCInfoText:SetTextColor(1,1,1,1)
end

-- Start
function AceGCInfo:Enable()
	self:RestorePosition()
	gnome:Register("AceGCInfo", self.UpdateText, 1, self)
	gnome:Start("AceGCInfo")
	AceGCInfoFrame:Show()
end

function AceGCInfo:Disable()
	gnome:Unregister("AceGCInfo")
	AceGCInfoFrame:Hide()
end

-- Events
-- Storage for stuff
--local previousMem, currentMem, gcThreshold, initMem

function AceGCInfo:UpdateData()
	self.previousMem = self.currentMem
	self.currentMem, self.gcThreshold = gcinfo()
end

function AceGCInfo:UpdateText()
	self:UpdateData()
	if (self.previousMem) then
		local str = format("Used: [%.1fMB] Threshold: [%.1fMB] Increase: [%dKB/s]",
				self.currentMem/1024,
				self.gcThreshold/1024,
				self.currentMem - self.previousMem
		)

		AceGCInfoText:SetText(str)
	end
end

function AceGCInfo:RestorePosition()
	local x = self.GetOpt("PosX")
	local y = self.GetOpt("PosY")
	if not x or not y then
		AceGCInfoFrame:ClearAllPoints()
		AceGCInfoFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		return
	end

	local s = AceGCInfoFrame:GetEffectiveScale()
	x,y = x/s, y/s
	
	AceGCInfoFrame:ClearAllPoints()
	AceGCInfoFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
end

function AceGCInfo:SavePosition()
	local x,y = AceGCInfoFrame:GetLeft(), AceGCInfoFrame:GetTop()
	local s = AceGCInfoFrame:GetEffectiveScale()

	x,y = x*s, y*s

	self.SetOpt("PosX", x)
	self.SetOpt("PosY", y)
end

-- Command handlers
function AceGCInfo:TogLock()
	self.TogOpt("Locked")
	self.Msg("Locking is now: "..ACEG_MAP_ONOFF[self.GetOpt("Locked") or 0])
end

function AceGCInfo:Report()
	self.cmd:report({
		{text="Locking:", val=self.GetOpt("Locked"), map=ACEG_MAP_ONOFF},
	})
end

-- FuBar functionality
if FuBar then
	function AceGCInFu:Initialize()
		self.tablet = TabletLib:GetInstance("1.0")
	end

	function AceGCInFu:Enable()
		AceGCInfo:Disable()
		gnome:Register(self.name, self["Update"], 1, self)
		gnome:Start(self.name)
		if not self.charData.visible then self.charData.visible = 3 end
	end

	function AceGCInFu:Disable()
		AceGCInfo:Enable()
		gnome:Unregister(self.name)
	end

	function AceGCInFu:UpdateData()
		AceGCInfo:UpdateData()
	end

	function AceGCInFu:UpdateText()
		if (AceGCInfo.previousMem) then
			local str

			if (self.charData.visible == 1) then
				str = format("Cur: [%.1fMB]", AceGCInfo.currentMem/1024)
			elseif (self.charData.visible == 2) then
				str = format("TH: [%.1fMB]", AceGCInfo.gcThreshold/1024)
			elseif (self.charData.visible == 3) then
				str = format("Inc: [%.1fKB/s]", AceGCInfo.currentMem - AceGCInfo.previousMem)
			end

			self:SetText(str)
		end
	end

	function AceGCInFu:UpdateTooltip()
		self.tablet:SetTitle("AceGCInFu")
		self.tablet:SetHint("Click to set bar text!")
		local cat = self.tablet:AddCategory("columns", 2)

		cat:AddLine(
			"text", "Current",
			"text2", string.format("%.1fMB", AceGCInfo.currentMem/1024),
			"text2R", 1,
			"text2G", 1,
			"text2B", 1,
			"func", function() self.charData.visible = 1 end
		)
		cat:AddLine(
			"text", "Threshold",
			"text2", string.format("%.1fMB", AceGCInfo.gcThreshold/1024),
			"text2R", 1,
			"text2G", 1,
			"text2B", 1,
			"func", function() self.charData.visible = 2 end
		)
		cat:AddLine(
			"text", "Increase",
			"text2", string.format("%.1fKB/s", AceGCInfo.currentMem - AceGCInfo.previousMem),
			"text2R", 1,
			"text2G", 1,
			"text2B", 1,
			"func", function() self.charData.visible = 3 end
		)
	end
end

-- Register
AceGCInfo:RegisterForLoad()
if FuBar then AceGCInFu:RegisterForLoad() end
